@@ -1,9 +1,9 @@
 ;;; find-file-in-project.el --- Find file/directory and review Diff/Patch/Commit efficiently everywhere
 
-;; Copyright (C) 2006-2009, 2011-2012, 2015, 2016, 2017
+;; Copyright (C) 2006-2009, 2011-2012, 2015-2018
 ;;   Phil Hagelberg, Doug Alcorn, Will Farrington, Chen Bin
 ;;
-;; Version: 5.6.8
+;; Version: 5.7.3
 ;; Author: Phil Hagelberg, Doug Alcorn, and Will Farrington
 ;; Maintainer: Chen Bin <chenbin.sh@gmail.com>
 ;; URL: https://github.com/technomancy/find-file-in-project
@@ -31,25 +31,24 @@
 
 ;;; Commentary:
 
-;; This program provides a couple methods for quickly finding any file
-;; in a given project.
-;; - Only dependency is GNU/BSD find
+;; This program provides methods to find file in project.
+;; - Only dependency is BSD/GNU find
 ;; - Works on Windows with minimum setup
-;; - Works flawlessly on Tramp Mode (https://www.emacswiki.org/emacs/TrampMode)
+;; - Works on Tramp Mode (https://www.emacswiki.org/emacs/TrampMode)
 ;; - fd (faster alternative of find, see https://github.com/sharkdp/fd) is supported
 ;;
 ;; Usage,
-;;   - You can insert `(setq ffip-use-rust-fd t)' into ".emacs" to use fd (alternative of find)
-;;   - `M-x find-file-in-project-at-point' guess the file name at point and
+;;   - You can insert "(setq ffip-use-rust-fd t)" into ".emacs" to use fd (alternative of find)
+;;   - `find-file-in-project-at-point' guess the file path at point and
 ;;      find file
-;;   - `M-x find-file-in-project-by-selected' use the selected region
-;;      as the keyword to search file.  Or you need provide the keyword
-;;      if no region selected.
-;;   - `M-x find-directory-in-project-by-selected' use the select region
-;;      to find directory.  Or you need provide the keyword if no region
-;;      selected.
-;;   - `M-x find-file-in-project' will start search file immediately
-;;   - `M-x ffip-create-project-file' create .dir-locals.el
+;;   - `find-file-in-project-by-selected' uses the selected region
+;;      as the keyword to search file.  You can provide the keyword
+;;      if no region is selected.
+;;   - `find-directory-in-project-by-selected' uses the select region
+;;      to find directory.  You can provide the keyword if no region
+;;      is selected.
+;;   - `find-file-in-project' will start search file immediately
+;;   - `ffip-create-project-file' creates ".dir-locals.el"
 ;;
 ;; A project is found by searching up the directory tree until a file
 ;; is found that matches `ffip-project-file'.
@@ -71,25 +70,27 @@
 ;; also used by `find-file-with-similar-name'.
 ;;
 ;; all these variables may be overridden on a per-directory basis in
-;; your .dir-locals.el.  See (info "(Emacs) Directory Variables") for
+;; your ".dir-locals.el".  See (info "(Emacs) Directory Variables") for
 ;; details.
 ;;
-;; Sample .dir-locals.el,
+;; Sample ".dir-locals.el",
 ;;
 ;; ((nil . ((ffip-project-root . "~/projs/PROJECT_DIR")
-;;          ;; ingore files bigger than 64k and directory "dist/"
+;;          ;; ignore files bigger than 64k and directory "dist/" when searching
 ;;          (ffip-find-options . "-not -size +64k -not -iwholename '*/dist/*'")
 ;;          ;; only search files with following extensions
 ;;          (ffip-patterns . ("*.html" "*.js" "*.css" "*.java" "*.xml" "*.js"))
 ;;          (eval . (progn
 ;;                    (require 'find-file-in-project)
-;;                    ;; ingore directory ".tox/"
-;;                    (setq ffip-prune-patterns `("*/.tox/*" ,@ffip-prune-patterns))
-;;                    ;; Do NOT ignore directory "bin/"
-;;                    (setq ffip-prune-patterns `(delete "*/bin/*" ,@ffip-prune-patterns))))
+;;                    ;; ignore directory ".tox/" when searching
+;;                    (setq ffip-prune-patterns `("*/.tox" ,@ffip-prune-patterns))
+                      ;; ignore BMP image file
+;;                    (setq ffip-ignore-filenames `("*.bmp" ,@ffip-ignore-filenames))
+;;                    ;; Do NOT ignore directory "bin/" when searching
+;;                    (setq ffip-prune-patterns `(delete "*/bin" ,@ffip-prune-patterns))))
 ;;          )))
 ;;
-;; To find in *current directory*, use `find-file-in-current-directory'
+;; To find in current directory, use `find-file-in-current-directory'
 ;; and `find-file-in-current-directory-by-selected'.
 ;;
 ;; `ffip-split-window-horizontally' and `ffip-split-window-vertically' find&open file
@@ -136,22 +137,17 @@
 ;;
 ;; You can switch to `ido-mode' by `(setq ffip-prefer-ido-mode t)'
 
-;; BSD/GNU Find can be installed through `Cygwin' or `MYSYS2' on Windows.
+;; BSD/GNU Find can be installed through Cygwin or MYSYS2 on Windows.
 ;; Executable is automatically detected. But you can manually specify
-;; the executable location by insert below code into ~/.emacs,
+;; the executable location by insert below code into ".emacs",
 ;;
 ;;   (if (eq system-type 'windows-nt)
 ;;      (setq ffip-find-executable "c:\\\\cygwin64\\\\bin\\\\find"))
 ;;
-;; This program works on Windows/Cygwin/Linux/Mac.
-;;
-;; Windows setup is as easy as installing Cygwin into default directory on
-;; ANY driver.
+;; This program works on Windows/Cygwin/Linux/macOS
 ;;
 ;; See https://github.com/technomancy/find-file-in-project for advanced tips.
 
-;; Recommended binding: (global-set-key (kbd "C-x f") 'find-file-in-project)
-
 ;;; Code:
 
 (require 'diff-mode)
@@ -190,7 +186,7 @@ It's used by `find-file-with-similar-name'.")
 The file path is passed to the hook as the first argument.")
 
 (defvar ffip-relative-path-pattern "^\\(\\.\\.*/\\)+"
-  "Pathern of relative path.")
+  "Pattern of relative path.")
 
 (defun ffip-shell-command-to-string (command)
   "Execute shell COMMAND and return its output as a string."
@@ -248,7 +244,7 @@ May be set using .dir-locals.el.  Checks each entry if set to a list.")
   "List of glob patterns to look for with `find-file-in-project'.")
 
 (defvar ffip-match-path-instead-of-filename nil
-  "Match full path instead of file name when calling `find-file-in-project-by-selected'.")
+  "Match full path instead of file name.")
 
 ;; For "GNU/BSD Find", "*/test/*" matches "./test/" and "./dir/test/"
 ;;
@@ -257,33 +253,47 @@ May be set using .dir-locals.el.  Checks each entry if set to a list.")
 ;; Maybe it's bug of fd.
 (defvar ffip-prune-patterns
   '(;; VCS
-    "*/.git/*"
-    "*/.svn/*"
-    "*/.cvs/*"
-    "*/.bzr/*"
-    "*/.hg/*"
+    "*/.git"
+    "*/.svn"
+    "*/.cvs"
+    "*/.tox"
+    "*/.bzr"
+    "*/.hg"
+    "*/bin"
+    "*/.DS_Store"
+    "*/.sass-cache"
+    "*/.npm"
+    "*/.tmp"
+    "*/.idea"
+    "*/node_modules"
+    "*/bower_components"
+    "*/.gradle"
+    "*/.cask")
+  "Ignored directories(prune patterns).")
+
+(defvar ffip-ignore-filenames
+  '(;; VCS
     ;; project misc
     "*.log"
-    "*/bin/*"
-    ;; Mac
-    "*/.DS_Store/*"
     ;; Ctags
-    "*/tags"
-    "*/TAGS"
+    "tags"
+    "TAGS"
+    ;; compressed
+    "*.tgz"
+    "*.gz"
+    "*.xz"
+    "*.zip"
+    "*.tar"
+    "*.rar"
     ;; Global/Cscope
-    "*/GTAGS"
-    "*/GPATH"
-    "*/GRTAGS"
-    "*/cscope.files"
+    "GTAGS"
+    "GPATH"
+    "GRTAGS"
+    "cscope.files"
     ;; html/javascript/css
-    "*/.npm/*"
-    "*/.tmp/*" ; TypeScript
-    "*/.sass-cache/*" ; SCSS/SASS
-    "*/.idea/*"
+    "*bundle.js"
     "*min.js"
     "*min.css"
-    "*/node_modules/*"
-    "*/bower_components/*"
     ;; Images
     "*.png"
     "*.jpg"
@@ -295,43 +305,47 @@ May be set using .dir-locals.el.  Checks each entry if set to a list.")
     ;; documents
     "*.doc"
     "*.docx"
+    "*.xls"
+    "*.ppt"
     "*.pdf"
+    "*.odt"
     ;; C/C++
     "*.obj"
+    "*.so"
     "*.o"
     "*.a"
+    "*.ifso"
+    "*.tbd"
     "*.dylib"
     "*.lib"
     "*.d"
     "*.dll"
     "*.exe"
     ;; Java
-    "*/.metadata*"
-    "*/.gradle/*"
+    ".metadata*"
     "*.class"
     "*.war"
     "*.jar"
     ;; Emacs/Vim
     "*flymake"
-    "*/#*#"
+    "#*#"
     ".#*"
     "*.swp"
     "*~"
     "*.elc"
-    "*/.cask/*"
     ;; Python
     "*.pyc")
-  "List of directory/file glob patterns to not descend into when listing files in `find-file-in-project'.")
+  "Ignore file names.  Wildcast is supported.")
 
 (defvar ffip-find-options ""
   "Extra options to pass to `find' when using `find-file-in-project'.
 
 Use this to exclude portions of your project: \"-not -regex \\\".*svn.*\\\"\".")
 
-(defcustom ffip-find-pre-path-options ""
+(defvar ffip-find-pre-path-options ""
   "Extra options to pass to `find' before path name options when using `find-file-in-project'.
 
-As required by `find', `-H', `-L', `-P', `-D' and `-O' must appear before the first path name, `.' for ffip's case.
+As required by `find', `-H', `-L', `-P', `-D' and `-O' must appear before the first path name, `.'.
 For example, use this to follow symbolic links inside your project: \"-L\".")
 
 (defvar ffip-project-root nil
@@ -345,10 +359,6 @@ This overrides variable `ffip-project-root' when set.")
 (defvar ffip-ivy-last-saved nil
   "Backup of `ivy-last'.  Requires ivy.")
 
-(defvar ffip-full-paths t
-  "If nil, search pattern only matches against file/directory name.
-If t,the pattern is matched against the full path.")
-
 (defvar ffip-debug nil "Print debug information.")
 
 ;;;###autoload
@@ -432,7 +442,7 @@ If t,the pattern is matched against the full path.")
 
 ;;;###autoload
 (defun ffip-filename-camelcase-to-dashes (keyword &optional check-only)
-  "Convert KEYWORD from camel cased to dash seperated.
+  "Convert KEYWORD from camel cased to dash separated.
 If CHECK-ONLY is true, only do the check."
   (let* (rlt)
     (cond
@@ -451,7 +461,7 @@ If CHECK-ONLY is true, only do the check."
 
 ;;;###autoload
 (defun ffip-filename-dashes-to-camelcase (keyword &optional check-only)
-  "Convert KEYWORD from dash seperated to camel cased.
+  "Convert KEYWORD from dash separated to camel cased.
 If CHECK-ONLY is true, only do the check."
   (let* (rlt)
     (cond
@@ -467,7 +477,7 @@ If CHECK-ONLY is true, only do the check."
     rlt))
 
 (defun ffip--create-filename-pattern-for-gnufind (keyword)
-  "Create search patthern from KEYWORD."
+  "Create search pattern from KEYWORD."
   (let* ((rlt ""))
     (cond
      ((not keyword)
@@ -542,19 +552,32 @@ If CHECK-ONLY is true, only do the check."
 
 (defun ffip--prune-patterns ()
   "Turn `ffip-prune-patterns' into a string that `find' can use."
+  ;; Both fd and find use "glob pattern"
+  ;; @see https://en.wikipedia.org/wiki/Glob_%28programming%29
   (cond
    (ffip-use-rust-fd
-    ;; rust use "regular expression" which matches part of string
-    ;; while find use "glob pattern" which matches the whole string
-    ;; @see https://en.wikipedia.org/wiki/Glob_%28programming%29
+    ;; fd match relative path
     (mapconcat (lambda (p)
                  (format "-E \"%s\"" (replace-regexp-in-string "^\*/" "" p)))
                ffip-prune-patterns " "))
    (t
+    ;; find match whole path
     (mapconcat (lambda (p)
                  (format "-iwholename \"%s\"" p))
                ffip-prune-patterns " -or "))))
 
+(defun ffip--ignore-file-names ()
+  "Turn `ffip-ignore-filenames' into a string that `find' can use."
+  ;; @see `ffip-prune-patterns' for fd vs find.
+  (cond
+   (ffip-use-rust-fd
+    (mapconcat (lambda (p)
+                 (format "-E \"%s\"" p))
+               ffip-ignore-filenames " "))
+   (t
+    (mapconcat (lambda (n) (format "-not -name \"%s\"" n))
+               ffip-ignore-filenames " "))))
+
 ;;;###autoload
 (defun ffip-completing-read (prompt collection &optional action)
   "Read a string in minibuffer, with completion.
@@ -602,14 +625,15 @@ BSD/GNU Find use glob pattern."
   (let* (cmd fmt tgt)
     (cond
      (ffip-use-rust-fd
+      ;; `-H` => search hidden files
       ;; `-E` => exclude pattern
       ;; `-c` => color
       ;; `-i` => case insensitive
       ;; `-t` => directory (d) or file (f)
       ;; `-p` => match full path
-      (setq fmt (concat "%s %s -c never -i -t %s %s %s"
-                        (if ffip-full-paths " -p" "")
+      (setq fmt (concat "%s %s -c never -H -i -t %s %s %s %s"
                         (if ffip-rust-fd-respect-ignore-files "" " -I")
+                        (if ffip-match-path-instead-of-filename " -p" "")
                         " "
                         ffip-rust-fd-extra-opts
                         " %s"))
@@ -621,14 +645,13 @@ BSD/GNU Find use glob pattern."
               (ffip--create-filename-pattern-for-gnufind keyword)))
       (setq fmt (concat "%s "
                         ffip-find-pre-path-options
-                        " . \\( %s \\) -prune -o -type %s %s %s %s -print"))))
+                        " . \\( %s \\) -prune -o -type %s %s %s %s %s -print"))))
 
     (setq cmd (format fmt
                       (ffip--executable-find)
                       (ffip--prune-patterns)
                       (if is-finding-directory "d" "f")
-                      ;; When finding directory, the keyword is like:
-                      ;; "proj/hello/world"
+                      (ffip--ignore-file-names)
                       ffip-find-options
                       (ffip--join-patterns ffip-patterns)
                       tgt))
@@ -651,8 +674,7 @@ If KEYWORD is string, it's the file name or file path to find file.
 If KEYWORD is list, it's the list of file names.
 IF IS-FINDING-DIRECTORY is t, we are searching directories, else files.
 DIRECTORY-TO-SEARCH specify the root directory to search."
-  (let* (rlt
-         (root (or directory-to-search
+  (let* ((root (or directory-to-search
                    (ffip-get-project-root-directory)))
          (default-directory (file-name-as-directory root))
          (cmd (ffip-create-shell-command keyword is-finding-directory))
@@ -666,16 +688,11 @@ DIRECTORY-TO-SEARCH specify the root directory to search."
         (setq collection (delq nil (mapcar (lambda (s)
                                              (if (string-match-p r s) s))
                                            collection)))))
-    (setq rlt
-          (mapcar (lambda (file)
-                    (if ffip-full-paths
-                        (cons (replace-regexp-in-string "^\./" "" file)
-                              (expand-file-name file))
-                      (cons (file-name-nondirectory file)
-                            (expand-file-name file))))
-                  ;; #15 improving handling of directories containing space
-                  collection))
-    rlt))
+    (mapcar (lambda (file)
+              (cons (replace-regexp-in-string "^\./" "" file)
+                    (expand-file-name file)))
+            ;; #15 improving handling of directories containing space
+            collection)))
 
 (defun ffip--forward-line (lnum)
   "Forward LNUM lines."
@@ -853,7 +870,7 @@ IF OPEN-ANOTHER-WINDOW is t, results are displayed in new window."
 
 ;;;###autoload
 (defun find-file-in-project-by-selected (&optional open-another-window)
-  "Same as `find-file-in-project' but more poweful and efficient.
+  "Same as `find-file-in-project' but more powerful and faster.
 It use string from selected region to search files in the project.
 If no region is selected, you could provide a keyword.
 
@@ -944,7 +961,7 @@ You can set `ffip-find-relative-path-callback' to format the string before copyi
 Use string from selected region to find directory in the project.
 If no region is selected, you need provide keyword.
 
-Keyword could be directory's base-name only or parent-directoy+base-name
+Keyword could be directory's base-name only or parent-directory+base-name
 For example, to find /home/john/proj1/test, below keywords are valid:
 - test
 - roj1/test
@@ -1089,7 +1106,7 @@ If OPEN-ANOTHER-WINDOW is not nil, the file will be opened in new window."
        ;; command
        ((functionp backend)
         (ffip-show-content-in-diff-mode (funcall backend)))
-       ;; lisp exipression
+       ;; lisp expression
        ((consp backend)
         (ffip-show-content-in-diff-mode (funcall `(lambda () ,backend)))))))
 
@@ -1103,7 +1120,7 @@ If OPEN-ANOTHER-WINDOW is not nil, the file will be opened in new window."
      ;; command
      ((functionp backend)
       (setq rlt (symbol-name backend)))
-     ;; lisp exipression
+     ;; lisp expression
      ((consp backend)
       ;; (cons "description" actual-backend)
       (if (stringp (car backend))
@@ -1113,7 +1130,7 @@ If OPEN-ANOTHER-WINDOW is not nil, the file will be opened in new window."
 
 ;;;###autoload
 (defun ffip-show-diff-internal (&optional num)
-  "Show the diff output by excuting selected `ffip-diff-backends'.
+  "Show the diff output by executing selected `ffip-diff-backends'.
 NUM is the index selected backend from `ffip-diff-backends'.
 NUM is zero based whose default value is zero."
   (interactive "P")
@@ -1131,7 +1148,7 @@ NUM is zero based whose default value is zero."
 
 ;;;###autoload
 (defun ffip-show-diff-by-description (&optional num)
-  "Show the diff output by excuting selected `ffip-diff-backends.
+  "Show the diff output by executing selected `ffip-diff-backends'.
 NUM is the backend index of `ffip-diff-backends'.
 If NUM is not nil, the corresponding backend is executed directly."
   (interactive "P")
@@ -1178,7 +1195,7 @@ If NUM is not nil, the corresponding backend is executed directly."
 (defun ffip-diff-apply-hunk (&optional reverse)
   "Apply current hunk in `diff-mode'.  Try to locate the file to patch.
 Similar to `diff-apply-hunk' but smarter.
-Please read documenation of `diff-apply-hunk' to get more details.
+Please read documentation of `diff-apply-hunk' to get more details.
 If REVERSE is t, applied patch is reverted."
   (interactive "P")
   (cond
@@ -1194,6 +1211,7 @@ If REVERSE is t, applied patch is reverted."
   (put 'ffip-diff-backends 'safe-local-variable 'listp)
   (put 'ffip-patterns 'safe-local-variable 'listp)
   (put 'ffip-prune-patterns 'safe-local-variable 'listp)
+  (put 'ffip-ignore-filenames 'safe-local-variable 'listp)
   (put 'ffip-match-path-instead-of-filename 'safe-local-variable 'booleanp)
   (put 'ffip-project-file 'safe-local-variable 'stringp)
   (put 'ffip-strip-file-name-regex 'safe-local-variable 'stringp)
